from cr_learn import ml_1m as ml
from cr_learn import beibei as bie
from cr_learn import ijcai   #no alias
from cr_learn import rees46
from cr_learn import tmall
from cr_learn import steam_games
from cr_learn import library_thing




__all__=[
    ml,
    bie,
    ijcai,
    rees46,
    tmall,
    steam_games,
    library_thing
]